# Steve Goldman, Space Telescope Science Institute, sgoldman@stsci.edu


class Fitting_Range_Error(ValueError):
    pass


class BadFilenameError(ValueError):
    pass


class BadSourceDirectoryError(ValueError):
    pass
